% FSSP3alg10_8_sub.m
%
%  This function subprogram detects a sinusoidal signal known except 
%  for amplitude, phase, and frequency (frequency not near 0 or 1/2)
%  embedded in WGN with known variance.  It computes the detection 
%  statistic and the threshold.
%
%  Input parameters:
%    x  - real array of dimension N x 1 containing data
%    sig2 - variance of WGN
%    Pfa  - probability of false alarm desired
%    Nfft - length of FFT used to compute periodogram, usually chosen as
%            2^nu for nu an integer 
%
%  Output parameters:
%    T   - detection statistic
%    gamma - threshold required
%   
function [T,gamma]=FSSP3alg10_8_sub(x,sig2,Pfa,Nfft)
N=length(x);
freq=[0:Nfft-1]'/Nfft;  % set up FFT frequencies over interval 0 <= f <1
I=(1/N)*abs(fft(x,Nfft)) .^2; % compute periodogram using FFT
T=max(I(1:Nfft/2-1));  % find maximum value over frequencies in interval
                       % 0<f<1/2
gamma=sig2*log(((N/2)-1)/Pfa);  % compute threshold

  
 
 
 
 
         
   
   
   
   
   
   
   
