% FSSP3alg9_10_sub.m
%
%  This program estimates the frequencies of real sinusoids by
%  using the principal component AR estimator.  
%
%  Input parameters:
%
%    x - Nx1 array of real data
%    p - number of sinusoids desired
%
%  Output parameters:
%
%    Ahat    - px1 real vector of amplitude estimates
%    phihat  - px1 real vector of phases estimates in radians
%    fhat    - px1 real vector of frequency estimates
%
function [Ahat,phihat,fhat]=FSSP3alg9_10_sub(x,p)
     N=length(x);
     L=floor(0.75*N); % set large AR model order
%  form forward-backward observation matrix H and data vector h 
     Hf=[];Hb=[];
     for i=1:N-L
       Hf=[Hf;flipud(x(i:i+L-1))'];
       Hb=[Hb;x(i+1:i+L)'];
     end
     H=[Hf;Hb];
     hf=x(L+1:N); hb=x(1:N-L);
     h=[hf;hb];
%  find the eigenvalues of H'H and arrange in descending order 
     d=flipud(sort(eig(H'*H)));
%  compute the principal component solution
%  use MATLAB's canned pseudoinverse solution 
     a=-pinv(H,0.5*(sqrt(d(2*p))+sqrt(d(2*p+1))))*h;
%  root the polynomial 
     z=roots([1;a]);
%  find the p roots closest to one in magnitude 
     dist=abs(abs(z)-1);
     [y,Index]=sort(dist);
%  use the phase angles of the roots as the frequency estimates 
       fhat(1:p,1)=abs(angle(z(Index(2*(1:p))))/(2*pi));
       fhat=sort(fhat);
%  determine the amplitudes and phases of the p sinusoids using least squares 
     n=[0:N-1]';  % set up time samples
     Hhat=[cos(2*pi*fhat(1)*n) sin(2*pi*fhat(1)*n)]; % construct estimate of 
                                                     % H matrix
     for i=2:p
       Hhat=[Hhat cos(2*pi*fhat(i)*n) sin(2*pi*fhat(i)*n)];
     end
     alpha=Hhat\x;   % find alpha's - this MATLAB operation is the same as
                     % using inv(H'*H)*H'*x but is numerically more
                     % stable
     i=(1:p); 
     Ahat(i,1)=sqrt(alpha(2*i-1) .^2+alpha(2*i) .^2); % final amplitude estimates
     phihat(i,1)=atan2(-alpha(2*i),alpha(2*i-1)); % final phase estimates
