%  FSSP3alg9_11_sub.m
%
%  This function subprogram extracts a periodic signal embedded in noise.
%
%  Input parameters:
%    x  - array of dimension N x 1 containing data samples
%    Mmin - minimum value for period
%    Mmax - maximum value for period
%    sig2   - variance of WGN
%
%  Output parameters:
%    shatr - estimated periodic signal
%    Mhat  - estimate of period
%
function [shatr,Mhat]=FSSP3alg9_11_sub(x,Mmin,Mmax,sig2)
   N=length(x);
   k=4; % compute periodogram for k*N samples.  The additional samples are
   % needed to ensure that the frequency samples at f=i/M for all the 
   % values of M are available.  See (9.20).
   I=(abs(fft(x,k*N)) .^2)/N;  % compute periodogram.
   Mmin=5;Mmax=50; % minimum and maximum period to search over
   J=zeros(Mmax,1);
   for M=Mmin:Mmax  % search J(M) over this range of M values
   f0hat=1/M;f=f0hat;  % set up fundamental frequency
   while f<0.5
   Iadd=I(round(f*k*N+1));  % pick up a peridogram sample at frequency
                            % 1/M,2/M,...,0.5
     if Iadd>sig2  % compute function to be maximized (9.20)
       J(M)=J(M)+Iadd/sig2-log(Iadd/sig2)-1;
     end
     f=f+f0hat;  % increase frequency to be added in next to function
     end   
   J(M)=J(M)-0.5*(M/2-1)*log(N); 
   end
   [Jmax Mhat]=max(J);  % find location of maximum to yield period 
                        % estimate Mhat
   Khat=floor(N/Mhat)+1; % find number of periods and round up
   shat=zeros(Mhat,1);  % this is basic period or ghat[n]
   for i=1:Mhat
       Kact=0;
   for k=1:Khat
       if i+(k-1)*Mhat<=N  % average data over total number of periods,
           % which can vary depending upon which s[n] we are estimating
           % if there are a noninteger number of periods
       shat(i)=shat(i)+x(i+(k-1)*Mhat);
       Kact=Kact+1;  % keep track of how many periods there are for each 
                     % estimated sample of s[n]
       end   
   end
   shat(i)=shat(i)/Kact;
   end
   kk=0;
   while kk<=N-1  % replicate basic signal period estimate to yield N points
       shatr(kk+1,1)=shat(mod(kk,Mhat)+1);
       kk=kk+1;
   end
 
   
   

