% FSSP3alg9_3_sub.m
%
%  This function subprogram estimates the amplitude, phase and frequency
%  of a sinusoid with known frequency embedded in WGN.  It uses the MLE.
%  It is assumed that the sinusoid is of the form A*cos(2*pi*f0*n+phi) 
%  for n=0,1,...,N-1.
%
%  Input parameters:
%    x  - real array of dimension Nx1 containing data
%
%  Output parameters:
%    Ahat  - estimate of amplitude
%    phihat - estimate of phase in radians
%    f0hat  - estimate of frequency in cycles/sample
%     
function [Ahat,phihat,f0hat]=FSSP3alg9_3_sub(x)
N=length(x);
n=[0:N-1]'; % set up time samples
  f0=[0.001:0.001:0.499]';  % set up frequency grid over which to maximize
                            % the function given by (9.6)
  for i=1:length(f0)  % compute the objective function for each f0
      arg=2*pi*f0(i)*n;
      H=[cos(arg) sin(arg)];
      J(i,1)=x'*H*inv(H'*H)*H'*x;
  end
  [maxJ I]=max(J);  % find the maximum of objective function and location 
                    % of the maximum 
  f0hat=f0(I);
arg=2*pi*f0hat*n;
H=[cos(arg) sin(arg)];  % set up H matrix using estimate of f0
alphahat=inv(H'*H)*H'*x;  % see (9.3)
Ahat=sqrt(alphahat(1)^2+alphahat(2)^2);  % see (9.2) for amplitude estimate
phihat=atan2(-alphahat(2),alphahat(1));  % see (9.2) for phase estimate


 
 

 

 
 
 
 
 
         
   
   
   
   
   
   
   
