%  FSSP3alg9_5_sub.m
%
%  This program estimates the arrival angle of a planar wave impinging
%  on a line array.  The arrival angle must satisfy 0<beta<pi/2, where
%  beta is measured from the x-axis or counterclockwise from the array axis.
%
%  Input parameters:
%    x  - real array of dimension M x 1 containing the spatial samples
%    F0 - frequency of sinusoidal signal in Hz
%    c  - propagation speed in m/s
%    d  - distance between adjacent sensors in m
%
%  Output parameters:
%    betahat - estimate of arrival angle in radians
%    beta - array containing the beta values for which the spatial 
%           periodogram is computed (if plotting is desired)
%    Is   - values of spatial periodogram (if plotting is desired)
%
function [betahat beta Is]=FSSP3alg9_5_sub(x,F0,c,d)
M=length(x);
n=[0:M-1]'; % set up spatial samples 
beta=[0.001:0.001:0.499]'*pi;  % set up angles to compute spatial periodogram
for i=1:length(beta)
    arg=-j*2*pi*F0*(d/c)*cos(beta(i))*n;
    Is(i,1)=(1/M)*abs(x'*exp(arg))^2;  % compute periodogram - see (9.9)
end
[maxIs II]=max(Is);  % find maximum and location of maximum of periodogram
betahat=beta(II);


 
 

 

 
 
 
 
 
         
   
   
   
   
   
   
   
