% FSSP3alg9_8_sub.m
%
%  This function subprogram estimates the center frequency of a bandpass
%  random signal whose power spectral density is known up to a shift, 
%  i.e, up to the center frequency, f0.  It is assumed that the true PSD 
%  is an autoregressive (AR) PSD embedded in WGN but the algorithm whereby 
%  J(f0) is minimized over f0 is valid for any PSD.
%
%  Input parameters:
%    x      - real array of dimension N x 1 containing data
%    alp    - real array of dimension p x 1 of lowpass AR filter parameters     
%    sig2u  - excitation white noise variance for AR process
%    B      - bandwidth of lowpass PSD
%    sig2   - variance of WGN
%
%   Output parameters:
%    f0hat - estimate of center frequency
%
%  External subprograms used: ARpsd.m, shift.m
%
function f0hat=FSSP3alg9_8_sub(x,alp,sig2u,B,sig2)

L=4096;  % number of PSD frequency samples to be computed
[Ps,f]=ARpsd(alp,sig2u,L);  % computes lowpass AR PSD, which is 
                            % symmetric, over L frequencies
                            % -1/2 < f < 1/2.  This is P_s(f).
N=length(x);
n=[0:N-1]';
    for i=1:L
    e=exp(j*2*pi*f(i)*n);
    Iper(i,1)=(1/N)*(abs(e'*x))^2;  % compute periodogram of data over
                                    % -1/2 <freq <1/2
    end
    f0min=B/2;f0max=0.5-B/2; % set minimum and maximum possible center
                             % frequencies for observed PSD 
    fshift=[round(L*f0min):round(L*f0max)]'; % convert possible values of
    % f0 to integers - used to shift Ps up in frequency to form P_s(f-f_0)
for i=1:length(fshift)
    powsdtry=shift(Ps,fshift(i))+sig2+0.001*max(Ps); % shift lowpass P_s up in 
    % frequency to form P_s(f-f_0) for each possible f_0 
    % Also, constant level of 0.001*max(Ps) needed to 
    % avoid a possible divide by zero if sig2=0
     J(i,1)=sum(Iper(L/2+1:L)./powsdtry(L/2+1:L))/L;  % use only the 
     % frequency samples over the interval 0<f<1/2 to compute integral -
     % see (9.15)
end
[mmin II]=min(J);  % find minimum of objective function and also location
                   % in frequency where minimum ocurrs
f0hat=fshift(II)/L; % estimate of center frequency













 
 
 
 
 
         
   
   
   
   
   
   
   
