% FSSP3alg9_9_sub.m
%
%  This program estimates the parameters for two sinusoids embedded in 
%  WGN using an exact MLE. 
%
%  Input parameters:
%    x - array of dimension N x 1 containing the data
%    delf - frequency spacing for grid search of maximimum of J(f1,f2)
%
%  Output parameters:
%    Ahat - array of dimension 2 x 1 containing amplitude estimates
%    phihat - array of dimension 2 x 1 containing phase estimates
%    fhat - array of dimension 2 x 1 containing frequency estimates
%
%  External programs required: mDmax.m
%
function [Ahat,phihat,fhat]=FSSP3alg9_9_sub(x,delf)
N=length(x); 
n=[0:N-1]';  % set up time samples
f=[delf:delf:0.5-delf]';  % set up grid of frequencies to search for 
                          % maximum of J(f1,f2)
J=zeros(length(f),length(f));
for i=1:length(f)  % for each value of (f1,f2) compute H(f) matrix
    c1=cos(2*pi*f(i)*n);
    s1=sin(2*pi*f(i)*n);
    H1=[c1 s1];
    for j=i+1:length(f)  % only compute H(f) for f2>f1
    c2=cos(2*pi*f(j)*n);
    s2=sin(2*pi*f(j)*n);
    H2=[c2 s2]; 
    H=[H1 H2];
    J(i,j)=x'*H*inv(H'*H)*H'*x;  % compute objective function - see (9.18)
    end 
end
[Jmax Index]=mDmax(J,2);  % find maximum and location of maximum of J(f1,f2)
f1hat=f(Index(1));  % convert maximum location of array to frequency estimates
f2hat=f(Index(2)); 
% once the frequencies estimates have been found, construct H(fhat) matrix
Hhat=[cos(2*pi*f1hat*n) sin(2*pi*f1hat*n) cos(2*pi*f2hat*n) sin(2*pi*f2hat*n)]; 
alphahat=inv(Hhat'*Hhat)*Hhat'*x; % find estimates of alpha's
A1hat=sqrt(alphahat(1)^2+alphahat(2)^2);
A2hat=sqrt(alphahat(3)^2+alphahat(4)^2);
phi1hat=atan2(-alphahat(2),alphahat(1));
phi2hat=atan2(-alphahat(4),alphahat(3));
Ahat=[A1hat A2hat]';  % final estimates of amplitudes
phihat=[phi1hat phi2hat]'; % final estimates of phases
fhat=[f1hat f2hat]'; % final estimates of frequencies




        
 








 
 
 
 
 
         
   
   
   
   
   
   
   
