% FSSP3exer11_3.m
%
% This program compares the MVSE (see (11.6)) for two different values of p. 
% It is assumed that the autocorrelation sequence is known, so that we
% are only assessing the potential resolution properties as p increases.
%
% External function subprograms required: 
%  ARpsd.m
%
clear all
close all
a=[0 0.9025]'; % AR filter parameters
sig2u=1; % AR excitation noise variance
p=50; % dimension of autocorrelation matrix
k=[0:p-1]';
r=0.95;
rx=(sig2u/(1-r^4))*(r.^k).*cos((pi/2)*k); % compute ACS
[powsd_true,freq]=ARpsd(a,sig2u,1024); % compute the true PSD
for i=1:p % compute known autocorrelation matrix for p=50
    for j=1:p
        Rx(i,j)=rx(abs(i-j)+1);
    end
end
freq=[0:1023]'/1024-0.5;
R1=Rx(1:10,1:10); % find known autocorrelation matrix for p=10
for i=1:1024 % compute MVSE for the two values of p
    e1=exp(sqrt(-1)*2*pi*freq(i)*[0:9]');
    P1(i,1)=10./real(e1'*inv(R1)*e1);
    e2=exp(sqrt(-1)*2*pi*freq(i)*k);
    P2(i,1)=50./real(e2'*inv(Rx)*e2);
end
plot(freq,10*log10(powsd_true),freq,10*log10(P1),freq,10*log10(P2))
grid
xlabel('f')
ylabel('MVSE PSD (dB)')
legend('true PSD','p=10','p=50')


        






