% FSSP3exer13_4.m
%
%  This program implements the MLE (see (13.6)) and autocorrelation 
%  (see (13.9)) frequency estimators. 
%
%  External function subprograms required:
%   ARgendata_complex.m
%
clear all
close all
randn('state',0) % initialize Gaussian random number generator
Q=15;
f0=[0.1:0.05:0.45]';  % set up values of center frequency to minimize
                      % objective function over
r=1./(1+(pi*f0)/Q); % relate the pole radius to the center frequency -
                    % see (13.3)
a1=-r.*exp(j*2*pi*f0); % set complex AR filter parameter
sig2u=1-abs(a1).^2; % set excitation white noise variance so that
                      % the average power in the AR process is 1
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%N=64;                  % data record length
N=32;                   % CHANGE DATA RECORD LENGTH
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
ntrials=10000;      % number of trials to use in Monte Carlo simulation
                    %  should increase number until the estimator
                    %  variance curve appears smooth
f0try=[0.05:0.001:0.5]'; % set up range for center frequency to be computed
% for objective function.  Note that it is slightly larger than range of
% known values of f0.  This is because otherwise, if the true value is say
% f0=0.1 and we compute the objective function starting at f0try=0.1, the
% error cannot be less than zero.  This causes a bias in the estimator and
% nullifies the use of the CRLB.  In practice, we seldom know exactly the
% range of f0 since it is usually a guess anyway. Furthermore, it makes
% the comparison with the periodogram fairer, which will search over
% [0,0.5] frequency range
rtry=1./(1+(pi*f0try)/Q); % range of possible pole radius
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%varw=0.01;
varw=0.1;  % CHANGE VARIANCE OF CWGN
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for k=1:length(f0)  % start loop over center frequencies   
for i=1:ntrials     % start loop over Monte Carlo trials
x=ARgendata_complex(a1(k),sig2u(k),N)+sqrt(varw)*randn(N,1);% generate complex AR signal
rho=sum(conj(x(1:N-1)).*x(2:N))/(x'*x);  % compute autocorrelation coefficient
                                         % see (13.8)
for l=1:length(f0try)  % start loop for objective function
J(l,1)=1+rtry(l)^2-2*rtry(l)*real(rho*exp(-j*2*pi*f0try(l))); % see (13.6)
end
[mm I]=min(J);% find minimum and location of minimum of objective function
f0hat(i,k)=f0try(I); % find MLE of center frequency
f0hat_ACS(i,k)=(1/(2*pi))*angle(rho);  % find the autcorrelation center
                                       % frequency estimator - see (13.9)
end
end
for k=1:length(f0)  % compute Monte Carlo variances
   var(k,1)=(1/ntrials)*sum( (f0hat(:,k)-mean(f0hat(:,k))).^2  ) ;
   var_ACS(k,1)=(1/ntrials)*sum( (f0hat_ACS(:,k)-mean(f0hat_ACS(:,k))).^2  ) ;
end

figure  % the rest of this code is for plotting and storing results
axes('position',[0.2,0.2,4*(0.375/3),4*(0.375/3)])
line(f0,var,'linewidth',1.5)  
line(f0,var_ACS,'linewidth',1.5,'linestyle','--')
grid
xlabel('$f_0$','interpreter','Latex')
ylabel('Variance')
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
legend('MLE','autocorrelation')  % ADD THIS TO LABEL CURVES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%axis([0.1 0.5 0 1e-4])
axis([0.1 0.5 0 1e-3])  % CHANGE AXIS SCALING
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
title('FSSP3exer13\_4')



