% FSSP3exer15_3.m
%
%  This program implements the AR spectral estimators using the covariance
%  method and the Burg method. The estimate is obtained for each of the 38
%  muscle noise segments and then plotted in an overlaid fashion as well
%  as the average of all the spectral estimates.
%
%  External function subprograms required:
%   PSD_est_AR_cov.m, PSD_est_AR_Burg.m
%
clear all
close all
load FSSP3exer15_3  % this loads in the data shown in Figure 15.6 as well
% as the start and end times for each segment.  Note that we must be
% careful to not use the data samples that were edited out since they 
% consist of all zeros.
x=noise;  % this is the muscle noise data shown in Figure 15.6
Fs=360; % sampling rate in samples/sec
plotit=0;  % don't plot the estimates but wait until all the 38 estimates
% have been computed
plotdB=0; % similarly for this parameter
Nf=4096; % number of frequencies over which to compute the spectral estimate
p=10; % model order for AR spectral estimate
PAR_avg=zeros(Nf,1); % set up array for the average of the estimates
nn=[start(1):endd(1)]'; % this is the first noise segment samples
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[f,PARdB]=PSD_est_AR_cov(x(nn),p,plotit,Nf,plotdB); % compute first 
% spectral estimate using covariance method
% Replace PSD_est_AR_cov with PSD_est_AR_Burg subprogram using the next line
% [f,PARdB]=PSD_est_AR_Burg(x(nn),p,plotit,Nf,plotdB); % compute first 
% spectral estimate using Burg method
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
PAR=10.^(PARdB/10); % convert back to linear quantities necessary to 
% average the estimates
plot(Fs*f(Nf/2:Nf/2+Nf/4),10*log10(PAR(Nf/2:Nf/2+Nf/4))) % plot first estimate
grid
xlabel('F (Hz)')
ylabel('AR PSD estimate (dB)')
title('overlaid estimates')
axis([0 90 -40 10]) % set up axes to match those shown in Figures 15.7 and 15.8
hold on  % hold the plot so that we can overlay the remaining 37 estimates
for i=2:length(start)
nn=[start(i):endd(i)]';
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[f,PARdB]=PSD_est_AR_cov(x(nn),p,plotit,Nf,plotdB); % compute 
% spectral estimate using covariance method
% Replace PSD_est_AR_cov with PSD_est_AR_Burg subprogram using the next line
% [f,PARdB]=PSD_est_AR_Burg(x(nn),p,plotit,Nf,plotdB); % compute  
% spectral estimate using Burg method
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
PAR=10.^(PARdB/10); % convert back to linear quantities necessary to 
% average the estimates
plot(Fs*f(Nf/2:Nf/2+Nf/4),10*log10(PAR(Nf/2:Nf/2+Nf/4))) % plot estimate
% overlaid with the others
PAR_avg=PAR_avg+PAR/38; % average the spectral estimates
end
hold off
figure
plot(Fs*f(Nf/2:Nf/2+Nf/4),10*log10(PAR_avg(Nf/2:Nf/2+Nf/4))) % plot average
% of the estimates
grid
xlabel('F (Hz)')
ylabel('AR PSD estimate (dB)')
title('average of the estimates')
axis([0 90 -40 10]) % set up axes to match those shown in Figures 15.7 and 15.8

        
    
    
    
   
    

        
    
    
   
        
        
    
    



