% FSSP3exer5_5.m
%
% This program generates a phase-modulated signal embedded in WGN.
% It then estimates all the parameters (linear and nonlinear) by 
% using a grid search for the nonlinear parameters.
% The external program mDmax.m is used to find the location of the 
% maximum and is included on the CD.
%
clear all
close all
randn('state',0)
varw=0.001; % change to varw=1 to reduce the SNR
varw=1;
N=100;A=1;phi=0;f0=0.1;m=0.0005; % set up parameters
n=[0:N-1]';
s=A*cos(2*pi*(f0*n+0.5*m*n.^2)+phi); % generate signal
x=s+sqrt(varw)*randn(N,1);  % add WGN
f0grid=[0.01:0.01:0.45]'; % set up grid of points to search for maximum
mgrid=[0.0001:0.0001:0.001]';
for i=1:length(f0grid)
    for j=1:length(mgrid)
        H=[cos(2*pi*(f0grid(i)*n+0.5*mgrid(j)*n.^2))...
            sin(2*pi*(f0grid(i)*n+0.5*mgrid(j)*n.^2))];  % compute observation matrix
        J(i,j)=x'*H*inv(H'*H)*H'*x;  % compute function to be maximized
    end
end
[fmax, Index]=mDmax(J,2); % find location of maximum
f0hat=f0grid(Index(1)) % this is the starting frequency estimate
mhat=mgrid(Index(2))   % this is the sweep rate estimate
% find estimates of amplitude and phase by substituting estimates of 
% starting frequency and sweep rate into observation matrix
Hhat=[cos(2*pi*(f0hat*n+0.5*mhat*n.^2)) sin(2*pi*(f0hat*n+0.5*mhat*n.^2))];
alphahat=inv(Hhat'*Hhat)*Hhat'*x;
Ahat=sqrt(alphahat'*alphahat) % this is estimate of amplitude
phihat=atan2(-alphahat(2),alphahat(1)) % this is estimate of phase
        
        

