% FSSP3exer7_5.m
%
% This program estimates the Pd versus SNR for a DC level in WGN. The
% theoretical performance can be found from (7.4).  Because the Pfa is so
% small we need a run the simulation under Ho using 1,000,000 trials.  It
% will take a long time to execute!  The Q and inverse Q functions are
% needed.  They are included on the CD as Q.m and Qinv.m, respectively.
%
clear all
close all
randn('state',0) % initialize random number generator
SNRdB=[-5:0.01:20]'; % set up SNR in dB
SNR=10.^(SNRdB/10);  % convert SNRs to linear quantities
Pfa=0.0001;
Pd=Q(Qinv(Pfa)-sqrt(SNR)); % compute theoretical Pd curve of (7.4)
figure
plot(SNRdB,Pd)  % plot theoretical Pd curve
grid
axis([-5 20 0 1.1])

 varw=5;  % WGN variance
 N=10;    % data record length
 for i=1:1000000
     x=sqrt(varw)*randn(N,1);  % generate outcomes of WGN
     T0(i,1)=mean(x);  % compute detection statistic
 end
T0_sort=sort(T0);  % sort into ascending order
gamma=T0_sort(999900);  % find threshold so that 100/1000000=0.0001 of the
                        % statistics exceed it
SNRdB=[-5:1:20]';  % repeat experiment but now add in signal
SNR=10.^(SNRdB/10);
  A=sqrt((varw*SNR)/N);  % compute DC level necessary for each SNR
  for k=1:length(A)
      for i=1:1000
          T1(i,1)=mean(A(k)+sqrt(varw)*randn(N,1));  % compute detection statistic
      end
      I=find(T1>gamma);  % find number of threshold crossings
      Pdest(k,1)=length(I)/1000;  %estimate Pd
  end
  figure
  plot(SNRdB,Pdest)  % plot Pd vs SNR curve
  grid
  xlabel('SNR (dB)')
  ylabel('Pd')
  axis([-5 20 0 1.1])
 

 
     
 






  
    