% FSSP3exer9_12.m
%
%  This program estimates the outcome of a random signal embedded in WGN
%  using a matrix Wiener smoother. See (9.21).  It assumes a Gaussian AR
%  random process which is stationary and so replaces the covariance
%  matrix by an autocorrelation matrix.
%
close all
clear all
randn('state',0);  % initialize Gaussian random number generator
a=-0.9;  % set up signal parameters
sig2u=1;
sig2=1;  % additive white noise variance
N=100;
n=[0:N-1]';  % set up time samples
s=ARgendata(a,sig2u,N);  % generate AR signal data
x=s+sqrt(sig2)*randn(N,1); %  add noise
figure  
plot(n,s) % plot the true signal
xlabel('n')
ylabel('s[n]')
title('True signal')
grid
axis([0 N -8 8]);
figure
plot(n,x) % plot the noise corrupted signal
xlabel('n')
ylabel('x[n]')
title('Noise corrupted signal')
grid
axis([0 N -8 8]);
r0=sig2u/(1-a^2);  % compute r_s[0]
rs=r0*(-a).^[0:N-1]';  % compute r_s[0],r_s[1],...,r_s[N-1]
shat=FSSP3alg9_12_sub(x,rs,sig2);  % call subprogram to estimate the outcome
                                   % of the AR signal process
figure
plot(n,shat)  % plot estimated signal
xlabel('n')
ylabel('shat[n]')
title('Estimated signal')
grid
axis([0 N -8 8]);









        

   
