% FSSP3fig14_8_exer.m
%
%  This program implements the Rao detector (see (14.8)), the energy 
%  detector (see (14.5)), and the Neyman-Pearson upper bound (see (14.4)) 
%  for the periodic signal shown in Figure 14.4 in IID Laplacian noise
%
%  External function subprograms required:
%   Laplacian_gendata.m, roccurve.m
%
clear all
close all
rand('state',0) % initalize uniform random number generator for Laplacian
                % noise generation
N=100;  % data record length
n=[0:N-1]';  % set up time samples
a=[1 0.5 0.5]';  % set amplitudes of cosine signals
b=a;             % set amplitudes of sine signals
f0=0.1;          % set true fundamental frequency
s=zeros(N,1);    % compute signal samples
for i=1:3
    s=s+a(i)*cos(2*pi*i*(0.1)*n)+b(i)*sin(2*pi*i*(0.1)*n);
end
f0=[0.09:0.001:0.11]';  % set up range of fundamental frequencies to search 
                        % for maximum 
ntrials=5000;           % number of Monte Carlo trials for performance
                        % evaluation
sig2=20;                % variance of Laplacian noise
for i=1:ntrials  
    w=Laplacian_gendata(sig2,N);  % generate Laplacian noise samples    
   x0=w;   % data set for noise only condition
   x1=s+w; % data set for signal plus noise condition
   y0=sign(x0);  % sign nonlinearity applied to data
   y1=sign(x1);
   for m=1:length(f0)  % compute sum of periodogram values at given
                       % fundamental frequency and first two harmonics
       T0(m,1)=(1/N)*abs(y0'*exp(-j*2*pi*f0(m)*n))^2+(1/N)*abs(y0'*exp(-j*4*pi*f0(m)*n))^2 ...
       +(1/N)*abs(y0'*exp(-j*6*pi*f0(m)*n))^2;  
       T1(m,1)=(1/N)*abs(y1'*exp(-j*2*pi*f0(m)*n))^2+(1/N)*abs(y1'*exp(-j*4*pi*f0(m)*n))^2 ...
       +(1/N)*abs(y1'*exp(-j*6*pi*f0(m)*n))^2;
   end
   TRao_0(i,1)=max(T0);  % Find maximum over all fundamental frequencies
   TRao_1(i,1)=max(T1);
   TNP_0(i,1)=sum(abs(x0)-abs(x0-s));  % compute NP upper bound detection
                                       % statistic
   TNP_1(i,1)=sum(abs(x1)-abs(x1-s));
   TED_0(i,1)=sum(x0'*x0); % compute energy detector statistic
   TED_1(i,1)=sum(x1'*x1);
   
end
ngam=50;  % set number of thresholds to compute (Pfa,Pd) operating point
[Pfa_NP,Pd_NP,gam]=roccurve(TNP_0,TNP_1,ngam);  % find (Pfa,Pd) for each
                                                % threshold for all
                                                % detectors
[Pfa_ED,Pd_ED,gam]=roccurve(TED_0,TED_1,ngam);
[Pfa_Rao,Pd_Rao,gam]=roccurve(TRao_0,TRao_1,ngam);

figure  % plot all the results
axes('position',[0.2,0.2,4*(0.375/3),4*(0.375/3)])
   line(Pfa_NP,Pd_NP,'linewidth',1.5)
   line(Pfa_ED,Pd_ED,'linewidth',1.5)
   line(Pfa_Rao,Pd_Rao,'linewidth',1.5)
  
  grid
  text(0.56,0.6,'Energy detector')
   text(0.32,0.7,'Rao detector')
  text(0.1,0.93,'NP upper bound')
xlabel('Pfa')
ylabel('Pd')
title('FSSP3fig14\_8\_exer.m')

   
   
  

