%  PSD_est_avper_complex.m
%
%  NOTE THAT THIS PROGRAM IS IDENTICAL TO PSD_est_avper.m SINCE MATLAB
%  REINTEPRETS ALL COMPLEX QUANTITIES CORRECTLY.  THE DATA VECTOR
%  x IS NOW A COMPLEX N x 1 VECTOR.
%
% This program computes the averaged periodogram spectral estimator.
% The number of periodograms I that will be averaged is the number of whole
% blocks of length L samples contained in the data array.
%
% Input parameters:
%
%   x    - Nx1 array of COMPLEX data
%   L    - length of each block to compute the individual periodograms
%   Nfft - length of FFT to compute periodogram
%   plotit - set equal to 1 to plot the PSD estimate
%   plotdB - set equal to 1 if vertical axis to be plotted in dB
%   yaxismin - minimum value of yaxis in dB (for autoscaling
%              leave blank)
%   yaxismax - maximum value of yaxis in dB (for autoscaling
%              leave blank)
%
%  Output parameters:
%
%   freq   - array of size Nfft x 1 containing the frequency samples
%   Pavper - array of size Nfft x 1 containing the periodogram values 
%
% Notes: use function [freq,Pavper]=PSD_est_avper(x,L,Nfft,plotit,plotdB,yaxismin,yaxismax) 
% to input the optional parameters yaxismin, yaxismax
%
  function [freq,Pavper]=PSD_est_avper_complex(x,L,Nfft,plotit,plotdB,varargin)
   N=length(x);
   I=floor(N/L);
   Pavper=zeros(Nfft,1);
   for i=0:I-1
       nstart=1+i*L;
       nend=L+i*L;
       y=x(nstart:nend,1);
   Pper=(1/L)*abs(fftshift(fft(y,Nfft))) .^2;  % compute periodogram and
   % shift halves of array so that they start at freq=-0.5 and end at
   % freq=0.5 as opposed to the [0,1] frequency interval used by FFT
   Pavper=Pavper+(1/I)*Pper;
   end
   freq=[0:Nfft-1]'/Nfft-0.5;  % compute frequency values
   if plotit==1
       if plotdB==1
       PavperdB=10*log10(Pavper);  % convert to dB quantities
       plot(freq,PavperdB)  % plot the averaged periodogram
       xlabel('frequency')
       ylabel('Averaged periodogram (dB)')
       grid
       else
       plot(freq,Pavper)  % plot the averaged periodogram
       xlabel('frequency')
       if I==1
       ylabel('Periodogram')
       else
       ylabel('Averaged periodogram')
       end
       grid
       end
   if nargin==6
      axis([-0.5 0.5 varargin{1} varargin{2}])
   end
   end
      
   


